#pragma once

#define MAX_RENDERABLE 20

#include <utils.h>
#include "WindowMapping.h"
#include "InfoDisplay.h"

/*
* Manages all layers as renderables, and as providers.
*/

class LayerManager
{
public:
	//please call this first.
	void Initialize(GLUI* parentGlui, int width, int height);

	void ResetCoords();
	void IncrementZoom(float delta);
	void ChangeZoom(float zoom);
	void ChangeParameter(float newParameter);
	void IncrementParameter(float delta);
	void MoveCoordinates(int x, int y);
	void SetCoordinates(float x, float y, float zoomX, float zoomY);

	static void event_RemoveLayer(int value);
	static void event_AddLayer(int value);

	void UpdateTextures();
	void Render();

	LayerManager(void);
	~LayerManager(void);

	WindowMapping m_windowMapping;

	InfoDisplay m_infoDisplay;

	//Layer Management:
	int AddLayer(Renderable * newLayer);
	void RemoveLayer();
	Renderable* m_renderableList[MAX_RENDERABLE];
	int m_renderableCount;


private:

	void UpdateWindowMapper();

	POINT_FLOAT m_coords;
	float m_zoomx, m_zoomy;
	int m_TotalLayerCount;

	int m_width, m_height;
	GLUI *m_parentGlui;
	GLUI_Panel *m_panelLayers;
	GLUI_Listbox *m_lbLayers;

	//only one - singleton-like
	static LayerManager *m_Manager;
};